'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _recompose = require('recompose');

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _shared = require('./shared');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var values = Object.values;


var withTransitionStates = (0, _recompose.withHandlers)(function () {
  var _leaving = {};
  var _justEntered = {};
  return {
    leaving: function leaving() {
      return function () {
        return _leaving;
      };
    },
    justEntered: function justEntered() {
      return function () {
        return _justEntered;
      };
    },
    setLeaving: function setLeaving() {
      return function (key, child, index) {
        _leaving[key] = { child: child, index: index };
      };
    },
    setLeft: function setLeft(_ref) {
      var forceUpdate = _ref.forceUpdate;
      return function (key) {
        delete _leaving[key];
        forceUpdate();
      };
    },
    setJustEntered: function setJustEntered(_ref2) {
      var forceUpdate = _ref2.forceUpdate;
      return function (key, bool) {
        if (bool) _justEntered[key] = bool;
        if (!bool) {
          delete _justEntered[key];
          forceUpdate();
        }
      };
    }
  };
});

var diffChildren = function diffChildren(prev) {
  var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref3$duration = _ref3.duration,
      duration = _ref3$duration === undefined ? 500 : _ref3$duration,
      setLeaving = _ref3.setLeaving,
      setLeft = _ref3.setLeft,
      setJustEntered = _ref3.setJustEntered,
      updates = _ref3.updates,
      children = _ref3.children;

  if (prev.updates === updates && prev.children === children) return false;

  var nextChildren = _react.Children.toArray(children).reduce(_shared.nextReducer, {});

  _react.Children.toArray(prev.children).forEach(function (child, i) {
    if (!nextChildren[child.key]) {
      setLeaving(child.key, child, i);
      setTimeout(function () {
        return setLeft(child.key);
      }, duration);
    } else {
      delete nextChildren[child.key];
    }
  });

  values(nextChildren).forEach(function (child) {
    setJustEntered(child.key, true);
    setTimeout(function () {
      return setJustEntered(child.key);
    }, 1);
  });

  return true;
};

var statusClassNames = function statusClassNames(_ref4) {
  var leaving = _ref4.leaving,
      justEntered = _ref4.justEntered,
      _ref4$before_enter = _ref4.before_enter,
      before_enter = _ref4$before_enter === undefined ? 'before_enter' : _ref4$before_enter,
      _ref4$entered = _ref4.entered,
      entered = _ref4$entered === undefined ? 'entered' : _ref4$entered,
      _ref4$after_exit = _ref4.after_exit,
      after_exit = _ref4$after_exit === undefined ? 'after_exit' : _ref4$after_exit,
      reverse = _ref4.reverse,
      children = _ref4.children,
      props = _objectWithoutProperties(_ref4, ['leaving', 'justEntered', 'before_enter', 'entered', 'after_exit', 'reverse', 'children']);

  return _extends({}, props, {
    children: _react.Children.map(children, function (child) {
      return leaving().hasOwnProperty(child.key) ? (0, _react.cloneElement)(child, { className: (0, _classnames2.default)(child.props.className, reverse ? before_enter : after_exit) }) : justEntered().hasOwnProperty(child.key) ? (0, _react.cloneElement)(child, { className: (0, _classnames2.default)(child.props.className, reverse ? after_exit : before_enter) }) : (0, _react.cloneElement)(child, { className: (0, _classnames2.default)(child.props.className, entered) });
    })
  });
};

exports.default = (0, _recompose.compose)(_shared.triggerForceUpdates, withTransitionStates, (0, _recompose.withPropsOnChange)(diffChildren, _shared.zipLeaving), (0, _recompose.mapProps)(statusClassNames));